local addonName, HideBattleNet = ... -- Get Addon Stuff
local L = HideBattleNet.L

--misc functions
local debugEnabled = false
local function debug(...)
	if debugEnabled then
		print(addonName .. " - " .. ...)
	end
end
debug("Debug Mode Enabled")

local function green(text)
	return "|cFF00FF00" .. text .. "|r"
end
local function red(text)
	return "|cFFFF0000" .. text .. "|r"
end

--Battle.Net friends
local OldBNFriends = BNGetNumFriends
local function NewBNFriends()
	if strmatch(debugstack(2),"FriendsList_Update") and HideBattleNet_Settings.DisableFriends then
		return 0, 0, 0, 0
	else 
		return OldBNFriends()
	end
end

--Quick Join
local quickJoinInitialized = false
local function InitQuickJoin()
	if not (QuickJoinToastButton) then return end
	local function qjtoggle(self)
		if HideBattleNet_Settings.DisableQuickJoin then 
			self:Hide()
		else 
			self:Show()
		end 
	end
	QuickJoinToastButton.Toast:HookScript("OnShow", qjtoggle)
	QuickJoinToastButton.Toast2:HookScript("OnShow", qjtoggle)

	local function qjtoggleframe(self)
		if string.match(FriendsFrameTab4:GetText(), QUICK_JOIN) and HideBattleNet_Settings.DisableQuickJoinTab then 
			FriendsFrameTab4:Hide()
		else 
			FriendsFrameTab4:Show()
		end 
	end
	FriendsFrame:HookScript("OnShow", qjtoggleframe)
	quickJoinInitialized = true
end

--Battle Tag
FriendsFrameBattlenetFrame:HookScript("OnShow", function(self)
	if HideBattleNet_Settings.DisableTagFrame then 
		--self.Tag:SetText("Hidden")
		self.Tag:SetAlpha(0)
		self.Tag:Hide()
		self:Hide()
	elseif HideBattleNet_Settings.DisableTag then 
		--self.Tag:SetText("Hidden")
		self.Tag:SetAlpha(0)
		self.Tag:Hide()
	else
		self.Tag:SetAlpha(1)
		self.Tag:Show()
		self:Show()
	end
end)

local function RefreshFrames()
	if not HideBattleNet_Settings.DisableTagFrame then 
		FriendsFrameBattlenetFrame:Show()
	end
	if not (QuickJoinToastButton) then return end
	if HideBattleNet_Settings.DisableQuickJoin then 
		QuickJoinToastButton:UnregisterEvent("SOCIAL_QUEUE_UPDATE")
		QuickJoinToastButton.Toast:Hide()
		QuickJoinToastButton.Toast2:Hide()
	else
		QuickJoinToastButton:RegisterEvent("SOCIAL_QUEUE_UPDATE")
		QuickJoinToastButton.Toast:Show()
		QuickJoinToastButton.Toast2:Show()
	end
end

--slash cmd
SLASH_HBN1 = '/hbn'
function SlashCmdList.HBN(msg)
	command, args = string.match(msg, "(.-)%s(.*)")
	if command == nil then
		command = msg
	end
	local message = L["ProperAddonName"] .. " - "
	if command == "tag" or command == "t" then
		message = message .. L["DisableTag"] .. ": "
		if (HideBattleNet_Settings.DisableTag) then
			HideBattleNet_Settings.DisableTag = false
			print(message .. red(L["Disabled"]))
		else
			HideBattleNet_Settings.DisableTag = true
			print(message .. green(L["Enabled"]))
		end
	elseif command == "friends" or command == "f" then
		if not (HideBattleNet_Settings.DisableFriendsFunction) then
			message = message .. L["DisableFriends"] .. ": "
			if (HideBattleNet_Settings.DisableFriends) then
				HideBattleNet_Settings.DisableFriends = false
				print(message .. red(L["Disabled"]))
			else
				HideBattleNet_Settings.DisableFriends = true
				print(message .. green(L["Enabled"]))
			end
		else
			print(message .. L["DisabledFriendsFunctionCommandError"])
		end
	elseif command == "quickjoin" or command == "qj" then
		if (QuickJoinToastButton) then
			message = message .. L["DisableQuickJoin"] .. ": "
			if (HideBattleNet_Settings.DisableQuickJoin) then
				HideBattleNet_Settings.DisableQuickJoin = false
				print(message .. red(L["Disabled"]))
			else
				HideBattleNet_Settings.DisableQuickJoin = true
				print(message .. green(L["Enabled"]))
			end
		else
			print(message .. L["QuickJoinNotFoundCommand"])
		end
	else
		print(L["HelpCommand"])
	end
	HideBattleNetPanel_Reload()
end

--interface options
function HideBattleNetPanel_OnLoad(panel)
	HideBattleNetPanel_Head:SetText(L["ProperAddonName"] .. " - " .. GetAddOnMetadata(addonName, "Version"));
    HideBattleNetPanel_DisableTagText:SetText(L["DisableTag"])
	HideBattleNetPanel_DisableTagFrameText:SetText(L["DisableTagFrame"])
	HideBattleNetPanel_DisableFriendsText:SetText(L["DisableFriends"])
	HideBattleNetPanel_DisabledFriendsFunctionText:SetText(L["DisabledFriendsFunction"])
	if QuickJoinToastButton then
		HideBattleNetPanel_DisableQuickJoinText:SetText(L["DisableQuickJoin"]);
		HideBattleNetPanel_DisableQuickJoinTabText:SetText(L["DisableQuickJoinTab"]);
	else
		HideBattleNetPanel_DisableQuickJoinText:SetText(red(L["QuickJoinNotFound"]));
		HideBattleNetPanel_DisableQuickJoinTabText:SetText(red(L["QuickJoinNotFound"]));
		HideBattleNetPanel_DisableQuickJoin:Disable()
		HideBattleNetPanel_DisableQuickJoinTab:Disable()
	end
	
	HideBattleNetPanel_BlizzardOptions:SetText(L["BlizzardOptions"])
	HideBattleNetPanel_BlizzardOptionsNote:SetText(L["BlizzardOptionsNote"])


	local function setoptiontext(panelframe, text, cvar)
		local onoroff = GetCVarBool(cvar) or false
		if (onoroff) then
			panelframe:SetText(text .. ": " .. green(L["Enabled"]))
		else
			panelframe:SetText(text .. ": " .. red(L["Disabled"]))
		end
	end
	setoptiontext(HideBattleNetPanel_ToastOnline, L["ToastOnline"], "showToastOnline")
	setoptiontext(HideBattleNetPanel_ToastOffline, L["ToastOffline"], "showToastOffline")
	setoptiontext(HideBattleNetPanel_ToastBroadcast, L["ToastBroadcast"], "showToastBroadcast")
	setoptiontext(HideBattleNetPanel_ToastFriendRequest, L["ToastFriendRequest"], "showToastFriendRequest")
	setoptiontext(HideBattleNetPanel_ToastWindow, L["ToastWindow"], "showToastWindow")

    panel.name = L["ProperAddonName"] -- category
    panel.okay = function (self) HideBattleNetPanel_Close(); end
    panel.cancel = function (self) HideBattleNetPanel_Close(); end
    InterfaceOptions_AddCategory(panel);
end

function HideBattleNetPanel_Close()
	HideBattleNet_Settings.DisableTag = HideBattleNetPanel_DisableTag:GetChecked()
	HideBattleNet_Settings.DisableTagFrame = HideBattleNetPanel_DisableTagFrame:GetChecked()
	HideBattleNet_Settings.DisableFriends = HideBattleNetPanel_DisableFriends:GetChecked()
	HideBattleNet_Settings.DisabledFriendsFunction = HideBattleNetPanel_DisabledFriendsFunction:GetChecked()
	HideBattleNet_Settings.DisableQuickJoin = HideBattleNetPanel_DisableQuickJoin:GetChecked()
	HideBattleNet_Settings.DisableQuickJoinTab = HideBattleNetPanel_DisableQuickJoinTab:GetChecked()
	
	RefreshFrames()
end

function HideBattleNetPanel_Reload()
    HideBattleNetPanel_DisableTag:SetChecked(HideBattleNet_Settings.DisableTag)
	HideBattleNetPanel_DisableTagFrame:SetChecked(HideBattleNet_Settings.DisableTagFrame)
	HideBattleNetPanel_DisabledFriendsFunction:SetChecked(HideBattleNet_Settings.DisabledFriendsFunction)
	if HideBattleNet_Settings.DisabledFriendsFunction then 
		HideBattleNetPanel_DisableFriends:SetChecked(false)
		HideBattleNetPanel_DisableFriends:Disable()
	else 
		HideBattleNetPanel_DisableFriends:SetChecked(HideBattleNet_Settings.DisableFriends)
		HideBattleNetPanel_DisableFriends:Enable()
	end
    HideBattleNetPanel_DisableQuickJoin:SetChecked(HideBattleNet_Settings.DisableQuickJoin)
	HideBattleNetPanel_DisableQuickJoinTab:SetChecked(HideBattleNet_Settings.DisableQuickJoinTab)
end

function HideBattleNetPanel_DisableFriendsFunction_Click()
	if HideBattleNetPanel_DisabledFriendsFunction:GetChecked() then
		HideBattleNetPanel_DisableFriends:SetChecked(false)
		HideBattleNetPanel_DisableFriends:Disable()
	else
		HideBattleNetPanel_DisableFriends:Enable()
	end
end

--events and loading
local frame = CreateFrame("FRAME", "HBN", UIParent)
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("FRIENDLIST_UPDATE")
frame:RegisterEvent("WHO_LIST_UPDATE")
frame:RegisterEvent("IGNORELIST_UPDATE")
frame:RegisterEvent("BN_BLOCK_LIST_UPDATED")
frame:RegisterEvent("BN_DISCONNECTED")
frame:RegisterEvent("BN_CONNECTED")
frame:RegisterEvent("BN_INFO_CHANGED")
frame:SetScript("OnEvent",function(self,event,...)
	if event == "PLAYER_ENTERING_WORLD" then
		HideBattleNetPanel_Reload()
		if quickJoinInitialized then
			RefreshFrames()
		else
			InitQuickJoin()
			RefreshFrames()
		end
	elseif event == "ADDON_LOADED" then
		frame:UnregisterEvent("ADDON_LOADED")

		--init settings
		if HideBattleNet_Settings == nil then
			HideBattleNet_Settings = {}
		end
		if HideBattleNet_Settings.DisableTag == nil then
			HideBattleNet_Settings.DisableTag = true
		end
		if HideBattleNet_Settings.DisableTagFrame == nil then
			HideBattleNet_Settings.DisableTagFrame = false
		end
		if HideBattleNet_Settings.DisableFriends == nil then
			HideBattleNet_Settings.DisableFriends = true
		end
		if HideBattleNet_Settings.DisabledFriendsFunction == nil then
			HideBattleNet_Settings.DisabledFriendsFunction = false
		end
		if HideBattleNet_Settings.DisableQuickJoin == nil then
			HideBattleNet_Settings.DisableQuickJoin = true
		end
		if HideBattleNet_Settings.DisableQuickJoinTab == nil then
			HideBattleNet_Settings.DisableQuickJoinTab = false
		end

		if HideBattleNet_Settings.DisabledFriendsFunction then
			HideBattleNet_Settings.DisableFriends = false
		else
			BNGetNumFriends = NewBNFriends
		end

		HideBattleNetPanel_Reload()
	elseif event == "BN_INFO_CHANGED" or "BN_CONNECTED" or "BN_DISCONNECTED" or "FRIENDLIST_UPDATE" or "WHO_LIST_UPDATE" or "IGNORELIST_UPDATE" or "BN_BLOCK_LIST_UPDATED" then
		--Battle tag safeguards
		if FriendsFrameBattlenetFrame then
			if HideBattleNet_Settings.DisableTagFrame then
				FriendsFrameBattlenetFrame.Tag:Hide()
				FriendsFrameBattlenetFrame.Tag:SetAlpha(0)
				FriendsFrameBattlenetFrame:Hide()
			elseif HideBattleNet_Settings.DisableTag then
				FriendsFrameBattlenetFrame.Tag:Hide()
				FriendsFrameBattlenetFrame.Tag:SetAlpha(0)
			end
		end
	end
end)